var lodash = require('lodash'),  //a node module used for filtering through arrays
    dataManagement = require('../data/dataManagement.js'), //for storing data in a json file
    validator = require('../data/schemaValidation.js'), //for validating the schema of the provider client wants to create
    uuidV1 = require('uuid/v1'), //for generating a random number for the id of the new providers
    localizify = require('localizify'), //for translation and localization for messages
    appInsights = require("applicationinsights"); //telemetry in the Azure


//error messages are default to English
const en = require('../resources/en.json');
localizify.add('en', en).setLocale('en');
const { t } = require('localizify');

//telemetry set up
appInsights.setup("4406b621-e8a9-44d1-8996-41906befaafd");
appInsights.start();
let client = appInsights.client;

var providerController = function (providerFilePath) {

    try {
        var providersJson = dataManagement.read(providerFilePath);
    }
    catch (err) {
        client.trackException(err);
    }

    //if the json file is corrupted or changed, delete all providers
    try {
        var providerList = JSON.parse(providersJson);
    }
    catch(err) {
        var providerList = [];
    }

    var postProvider = function (req, res) {

        /// <summary>Creates a new provider on the server</summary>  
        /// <param name="req" type="Object">The request sent to the server</param>  
        /// <param name="res" type="Object">The response sent back to the client</param>  

        if (req.body == null) {
            res.status(400);
            res.send(t('no_body_input'));
            return;
        }

        //random id of each provider
        req.body.id = uuidV1();

        var output = lodash.filter(providerList, x => x.name == req.body.name);
        if (output.length != 0 && req.body.name != null) {
            res.status(400);
            res.send(t('existing_provider'));
            return;
        }


        //try to validate Schema
        try {
            validator.validateProvider(req.body);
            providerList.push(req.body);

            //try to write data 
            try {
                dataManagement.write(providerFilePath, JSON.stringify(providerList));
                res.status(201);
                res.send(t('new_provider_added'));
            }
            catch (err) {
                client.trackException(err);
                res.status(400);
                res.send(err.message);
            }
        }
        catch (err) {
            client.trackException(err);
            res.status(400);
            res.send(err.message);
        }

    }

    var getProviderByName = function (req, res) {

        /// <summary>Get a specific provider from the server by name</summary>  
        /// <param name="req" type="Object">The request sent to the server</param>  
        /// <param name="res" type="Object">The response sent back to the client</param>  

        var name = req.params.name;

        var output = lodash.filter(providerList, x => x.name == name);

        if (output.length <= 0 || providerList.length == 0) {
            res.status(400);
            res.send(t('no_provider_with_name'));
            return;
        }
        res.json(output);
        res.status(200);
    }

    var getProviderList = function (req, res) {

        /// <summary>Get all of the providers from the server</summary>  
        /// <param name="req" type="Object">The request sent to the server</param>  
        /// <param name="res" type="Object">The response sent back to the client</param> 

        res.status(200);
        res.json(providerList);
    }

    var putProvider = function (req, res) {

        /// <summary>Updates an existing provider in the server</summary>  
        /// <param name="req" type="Object">The request sent to the server</param>  
        /// <param name="res" type="Object">The response sent back to the client</param> 

        var putIndex = providerList.map(function (item) { return item.name; })
            .indexOf(req.body.name);
        if (putIndex == -1) {
            res.status(400);
            res.send(t('no_provider_with_name'));
            return;
        }

        if (!req.body || req.body.length == 0) {
            res.status(400);
            res.send(t('no_body_input'));
            return;
        }

        if (req.body.id != null) {
            res.status(400);
            res.send(t('cannot_change_id'));
            return;
        }

        providerList[putIndex].name = req.body.name;
        providerList[putIndex].company = req.body.company;
        providerList[putIndex].url = req.body.url;
        providerList[putIndex].logo = req.body.logo;
        providerList[putIndex].plan = req.body.plan;

        try {
            dataManagement.write(providerFilePath, JSON.stringify(providerList));
            res.status(201);
            res.json(providerList[putIndex]);
        }
        catch (err) {
            client.trackException(err);
            res.status(400);
            res.send(err.message);
        }
    }

    var deleteProvider = function (req, res) {

        /// <summary>Deletes a specific provider from the server by name</summary>  
        /// <param name="req" type="Object">The request sent to the server</param>  
        /// <param name="res" type="Object">The response sent back to the client</param> 

        var removeIndex = providerList.map(function (item) { return item.name; })
            .indexOf(req.body.name);

        if (removeIndex == -1) {
            res.status(400);
            res.send(t('no_provider_with_name'));
            return;
        }

        res.status(200);
        res.send(t('provider_deleted'));

        //deletes provider and returns the object in res.json
        ~removeIndex && providerList.splice(removeIndex, 1);

        try {
            dataManagement.write(providerFilePath, JSON.stringify(providerList));
        }
        catch (err) {
            client.trackException(err);
            res.status(400);
            res.send(err.message);
        }
    }

    return {
        postProvider: postProvider,
        getProviderByName: getProviderByName,
        putProvider: putProvider,
        deleteProvider: deleteProvider,
        getProviderList: getProviderList
    }


}

module.exports = providerController;