var should = require('should'),
    sinon = require('sinon'),
    dataManagement = require('../data/dataManagement.js'),
    Provider = require('../models/Provider.js');

var providerControllerFile = '../controllers/providerController.js';
var testJSON = './data/testproviders.json';

describe('Provider Controller Tests:', function () {
    describe('Post Tests', function () {
        it('should not allow an empty name on post', function () {

            var providerController = require(providerControllerFile)(testJSON);

            var req = {
                body: {
                    company: "test",
                    url: "http://test.com"
                }
            }

            var res = {
                status: sinon.spy(),
                send: sinon.spy()
            }
           
            providerController.postProvider(req, res);

            //here is the status that this was called with
            res.status.calledWith(400).should.equal(true, 'Bad Status ' + res.status.args[0][0]);

            res.send.calledWith('instance.name is required').should.equal(true);
        })
        it('should allow a first time post', function () {

            var providerController = require(providerControllerFile)(testJSON);

            var req = {
                body: {
                    name: "Test Service",
                    company: "Test",
                    url: "http://test.com"
                }
            }

            var res = {
                status: sinon.spy(),
                send: sinon.spy()
            }

            providerController.postProvider(req, res);
            //here is the status that this was called with
            res.status.calledWith(201).should.equal(true, res.status.args[0][0]);
            res.send.calledWith('New Provider Added').should.equal(true);

           
        })
        it('should allow a second time post', function () {

            var provider = {

                name: "Already Registered Provider",
                company: "Already",
                url: "http://test.com",
                id: "5"
            }
            var array = []
            array.push(provider);

            dataManagement.write(testJSON, JSON.stringify(array));
            var providerController = require(providerControllerFile)(testJSON);

            var req = {
                body: {
                    name: "Test Service",
                    company: "Test",
                    id: "4",
                    url: "http://test.com"
                }
            }

            var res = {
                status: sinon.spy(),
                send: sinon.spy()
            }

            providerController.postProvider(req, res);
            //here is the status that this was called with
            res.status.calledWith(201).should.equal(true, res.status.args[0][0]);
            res.send.calledWith('New Provider Added').should.equal(true);


        })
        it('should not allow a second time post of same name', function () {

            var provider = {

                name: "Already Registered Provider",
                company: "Test",
                id: "5",
                url: "http://test.com"
            }

            var array = []
            array.push(provider);

            dataManagement.write(testJSON, JSON.stringify(array));
            var providerController = require(providerControllerFile)(testJSON);

            var req = {
                body: {
                    name: "Already Registered Provider",
                    company: "Test",
                    id: "4",
                    url: "http://test.com"
                }
            }

            var res = {
                status: sinon.spy(),
                send: sinon.spy()
            }

            providerController.postProvider(req, res);
            //here is the status that this was called with
            res.status.calledWith(400).should.equal(true, 'BAD STATUS ' + res.status.args[0][0]);
            res.send.calledWith('There already exists a provider with that name!').should.equal(true);
        })
        it('should not allow a post of same name', function () {

            createProviderTestFile(10);
            var providerController = require(providerControllerFile)(testJSON);

            var req = {
                body: {
                    name: "Test Provider 4",
                }
            }

            var res = {
                status: sinon.spy(),
                send: sinon.spy()
            }

            providerController.postProvider(req, res);
            //here is the status that this was called with
            res.status.calledWith(400).should.equal(true, 'BAD STATUS ' + res.status.args[0][0]);
            res.send.calledWith('There already exists a provider with that name!').should.equal(true);
        })
        it('should not allow a post with empty request body', function () {

            createProviderTestFile(10);
            var providerController = require(providerControllerFile)(testJSON);

            var req = {
                body: {
                }
            };

            var res = {
                status: sinon.spy(),
                send: sinon.spy()
            }

            providerController.postProvider(req, res);
            //here is the status that this was called with
            res.status.calledWith(400).should.equal(true, 'BAD STATUS ' + res.status.args[0][0]);
            res.send.calledWith('instance.name is required,instance.company is required').should.equal(true);
        })
        it('should not allow a post with null request body', function () {

            createProviderTestFile(10);
            var providerController = require(providerControllerFile)(testJSON);

            var req = {
                body: null
            };

            var res = {
                status: sinon.spy(),
                send: sinon.spy()
            }

            providerController.postProvider(req, res);
            //here is the status that this was called with
            res.status.calledWith(400).should.equal(true, 'BAD STATUS ' + res.status.args[0][0]);
            res.send.calledWith('No Body Input').should.equal(true);
        })

       afterEach(function () {
           dataManagement.write(testJSON, "");
        });
    })

    describe('Get List of Providers Tests', function () {
        it('should get a list of providers', function () {

            var provider = {

                name: "Already Registered Provider",
                company: "Test",
                id: "5",
                url: "http://test.com"
            }

            var array = []
            array.push(provider);

            dataManagement.write(testJSON, JSON.stringify(array));

            var providerController = require(providerControllerFile)(testJSON);
            var fileString = dataManagement.read(testJSON);

            var req = sinon.spy();

            var res = {
                status: sinon.spy(),
                send: sinon.spy(),
                json: sinon.spy()
            }

            providerController.getProviderList(req, res);

            res.status.calledWith(200).should.equal(true, res.status.args[0][0]);
            res.json.calledWith(JSON.parse(fileString)).should.equal(true);
        })
        it('should get a empty list of providers', function () {

            var providerController = require(providerControllerFile)(testJSON);
            var fileString = dataManagement.read(testJSON);
            var req = sinon.spy();
            var res = {
                status: sinon.spy(),
                send: sinon.spy(),
                json: sinon.spy()
            }

            providerController.getProviderList(req, res);

            res.status.calledWith(200).should.equal(true, res.status.args[0][0]);
        })
        afterEach(function () {
            dataManagement.write(testJSON, "");
        });
    })

    describe('Get Provider By Name Tests', function () {
        it('should get a provider', function () {

            var provider = {

                name: "Registered Provider",
                company: "Test",
                id: "5",
                url: "http://test.com"
            }

            var array = []
            array.push(provider);

            dataManagement.write(testJSON, JSON.stringify(array));

            var providerController = require(providerControllerFile)(testJSON);

            var req = {

                params: {
                    name: "Registered Provider"
                }
            };

            var res = {
                status: sinon.spy(),
                send: sinon.spy(),
                json: sinon.spy()
            }

            providerController.getProviderByName(req, res);

            res.status.calledWith(200).should.equal(true);
            res.json.firstCall.args[0][0].name.should.equal("Registered Provider");
            res.json.firstCall.args[0][0].url.should.equal("http://test.com");
        })
        it("should get an error: 'No Provider with that name!'", function () {

            var providerController = require(providerControllerFile)(testJSON);
            var fileString = dataManagement.read(testJSON);

            var req = {

                params: {
                    name: "Registered Provider"
                }
            };

            var res = {
                status: sinon.spy(),
                send: sinon.spy(),
                json: sinon.spy()
            }

            providerController.getProviderByName(req, res);

            res.status.calledWith(400).should.equal(true, res.status.args[0][0]);
            res.send.calledWith("No Provider with that name!").should.equal(true);
        })
        it('should get provider from a list of ten providers', function () {

            createProviderTestFile(10);

            var providerController = require(providerControllerFile)(testJSON);
            var fileString = dataManagement.read(testJSON);

            var req = {

                params: {
                    name: "Test Provider 3"
                }
            };

            var res = {
                status: sinon.spy(),
                send: sinon.spy(),
                json: sinon.spy()
            }

            providerController.getProviderByName(req, res);

            res.status.calledWith(200).should.equal(true, res.status.args[0][0]);
            res.json.firstCall.args[0][0].name.should.equal("Test Provider 3");
        })
        afterEach(function () {
            dataManagement.write(testJSON, "");
        });
    })

    describe('Delete Provider', function () {
        it('should delete a provider', function () {

            var provider = {

                name: "Registered Provider",
                company: "Test",
                id: "5",
                url: "http://test.com"
            }

            var array = []
            array.push(provider);

            dataManagement.write(testJSON, JSON.stringify(array));

            var providerController = require(providerControllerFile)(testJSON);

            var req = {

                body: {
                    name: "Registered Provider"
                }
            };

            var res = {
                status: sinon.spy(),
                send: sinon.spy(),
                json: sinon.spy()
            }

            providerController.deleteProvider(req, res);

            res.status.calledWith(200).should.equal(true);
            res.send.calledWith("Provider Deleted").should.be.equal(true);

            var list = JSON.parse(dataManagement.read(testJSON));
            list.length.should.equal(0);
        })
        it("should get an error: 'No Provider with that name!'", function () {

            var providerController = require(providerControllerFile)(testJSON);
            var fileString = dataManagement.read(testJSON);

            var req = {

                body: {
                    name: "Registered Provider"
                }
            };

            var res = {
                status: sinon.spy(),
                send: sinon.spy(),
                json: sinon.spy()
            }

            providerController.deleteProvider(req, res);

            res.status.calledWith(400).should.equal(true, res.status.args[0][0]);
            res.send.calledWith("No Provider with that name!").should.equal(true);
        })
        it('should delete a provider from a list of ten providers', function () {

            createProviderTestFile(10);

            var providerController = require(providerControllerFile)(testJSON);
            var fileString = dataManagement.read(testJSON);

            var req = {

                body: {
                    name: "Test Provider 3"
                }
            };

            var res = {
                status: sinon.spy(),
                send: sinon.spy()
            }

            providerController.deleteProvider(req, res);

            res.status.calledWith(200).should.equal(true);
            res.send.calledWith("Provider Deleted").should.be.equal(true);

            var list = JSON.parse(dataManagement.read(testJSON));
            list.length.should.equal(9);

        })
    })

    describe('Put Provider', function () {
        it('should alter a provider', function () {

            var provider = {

                name: "Registered Provider",
                url: "http://test.com",
                company: "Test",
                id: "5"
            }

            var array = []
            array.push(provider);

            dataManagement.write(testJSON, JSON.stringify(array));

            var providerController = require(providerControllerFile)(testJSON);

            var req = {

                body: {
                    name: "Registered Provider",
                    url: "http://different.com"
                }
            };

            var res = {
                status: sinon.spy(),
                send: sinon.spy(),
                json: sinon.spy()
            }

            providerController.putProvider(req, res);

            res.status.calledWith(201).should.equal(true);
            res.json.firstCall.args[0].name.should.equal("Registered Provider");
            res.json.firstCall.args[0].url.should.equal("http://different.com");
        })
        it("should get an error: 'No Provider with that name!'", function () {

            var providerController = require(providerControllerFile)(testJSON);
            var fileString = dataManagement.read(testJSON);

            var req = {

                body: {
                    name: "Registered Provider"
                }
            };

            var res = {
                status: sinon.spy(),
                send: sinon.spy(),
                json: sinon.spy()
            }

            providerController.putProvider(req, res);

            res.status.calledWith(400).should.equal(true, res.status.args[0][0]);
            res.send.calledWith("No Provider with that name!").should.equal(true);
        })
        it('should alter a provider from a list of ten providers', function () {

            createProviderTestFile(10);

            var providerController = require(providerControllerFile)(testJSON);
            var fileString = dataManagement.read(testJSON);

            var req = {

                body: {
                    name: "Test Provider 3",
                    url: "http://different.com",
                    logo: "a logo",
                    plan: "new plan"
                }
            };

            var res = {
                status: sinon.spy(),
                send: sinon.spy(),
                json: sinon.spy()
            }

            providerController.putProvider(req, res);

            res.status.calledWith(201).should.equal(true);
            res.json.firstCall.args[0].name.should.equal("Test Provider 3");
            res.json.firstCall.args[0].url.should.equal("http://different.com");
            res.json.firstCall.args[0].logo.should.equal("a logo");
            res.json.firstCall.args[0].plan.should.equal("new plan");

        })
        afterEach(function () {
            dataManagement.write(testJSON, "");
        });
    })
})


//helper functions that fill up testJSON file with Providers
var stringOfProviders = function (number) {

    var array = [];

    for (i = 1; i <= number; i++) {
        provider = {
            name: "Test Provider " + i,
            company: "Test",
            id: "" + i
        };

        array.push(provider);
    }
    return JSON.stringify(array);
};

var createProviderTestFile = function (number) {
    var str = stringOfProviders(number);
    dataManagement.write(testJSON, str);
};